class ProductExecutorCtor:
    def __init__(self, io, params, values, tempParams, ui):
        self.io = io
        self.params = params
        self.values = values
        self.tempParams = tempParams
        self.ui = ui